import * as React from 'react';
import * as _ from 'lodash';
// import update from 'immutability-helper';

import {
  Checkbox,
  Container,
  Label,
  Accordion,
  Icon,
  List, Button,
  ListItemProps,
  CheckboxProps,
  Rating,
} from 'semantic-ui-react';


import MetaSensorItemType from '../../api/model/MetaSensorItemType';
import MetaSensorItem from '../../api/model/MetaSensorItem';
import MetaSensorDisplayItem from '../../api/model/MetaSensorDisplayItem';
import MetaCrawler from '../../api/model/MetaCrawler';

import Sensor from '@overflow/sensor/api/model/Sensor';
import SensorItem from '@overflow/sensor/api/model/SensorItem';

import * as Utils from '@overflow/commons/util/Utils';
import SensorRegistInfo from '@overflow/sensor/api/model/SensorRegistInfo';

export interface SensorItemTreeStateProps {
  crawlerId?: number;

  // sensorItemList?: SensorItem[];
  // metaSensorItemTypeList?: MetaSensorItemType[];
  // metaSensorItemList?: MetaSensorItem[];
  // metaSensorItemKeyList?: MetaSensorItemKey[];

  sensorDisplayItems?: MetaSensorDisplayItem[];
  setSensor?(data: SensorRegistInfo): void;
  getSensor?(): SensorRegistInfo;
}

export interface SensorItemTreeDispatchProps {
  // onReadAllBySensor?(sensor: Sensor): void;
  // onSensorItemTypeReadAll?(): void;
  // onReadAll?(): void;
  // onReadAllSensorItemKeyByCrawler?(metaCrawler: MetaCrawler): void;

  onReadAllDisplayItemByCrawler?(metaCrawler: MetaCrawler): void;
}


export type SensorItemTreeProps = SensorItemTreeStateProps & SensorItemTreeDispatchProps;


export interface SensorItemTreeState {
  categoryState: Map<number, boolean>;
  categoryCheckState: Map<number, boolean>;
  itemState: Map<number, Array<boolean>>;

}

interface TreeItem {
  metaSensorDisplayItems: Array<MetaSensorDisplayItem>;
  metaSensorItemType: MetaSensorItemType;
}

export class SensorItemTree extends React.Component<SensorItemTreeProps, SensorItemTreeState> {

  private itemMap: Map<number, TreeItem>;
  private selectedItemMap: Map<number, Array<MetaSensorItem>>;

  constructor(props: any, context: any) {
    super(props, context);

    this.itemMap = new Map;
    this.selectedItemMap = new Map,

      this.state = {
        categoryState: new Map(),
        categoryCheckState: new Map(),
        itemState: new Map(),
      };

    this.props.onReadAllDisplayItemByCrawler({ id: this.props.crawlerId });
  }


  public ViewSensorItemType(): JSX.Element[] {

    if (this.props.sensorDisplayItems === undefined) {
      return null;
    }

    this.sortItemMap();

    let elems: JSX.Element[] = new Array();

    this.itemMap.forEach((data: TreeItem, key: number) => {
      elems.push(
        <List.Item key={key} >
          <List.Icon name={this.state.categoryState[key] ? 'chevron up' : 'chevron down'}
            onClick={this.onTypeClick.bind(this, key)} />
          <List.Content>
            <List.Header>
              <Checkbox label={data.metaSensorItemType.name} checked={this.state.categoryCheckState[key]} onChange={
                (event: React.FormEvent<HTMLInputElement>, checkProps: CheckboxProps) => {
                  this.onTypeCheckCBox(key, checkProps, data);
                }} />
            </List.Header>
            <List.Description style={{ marginLeft: '26px' }}></List.Description>
            {this.state.categoryState[key] ? this.ViewSensorItem(data.metaSensorDisplayItems, key) : ''}
          </List.Content>
        </List.Item>,
      );
    });

    return elems;
  }



  public ViewSensorItem(metaSensorDisplayItemList: MetaSensorDisplayItem[], parentKey: number): JSX.Element[] {
    let elems: JSX.Element[] = new Array();

    if (this.state.itemState[parentKey] === undefined) {
      this.state.itemState[parentKey] = new Array(metaSensorDisplayItemList.length).fill(false);
    }

    metaSensorDisplayItemList.map((item: MetaSensorDisplayItem, idx: number) => {
      elems.push(
        <List.List key={String(parentKey) + String(idx)} onClick={this.onItemClick.bind(this, parentKey, idx)}>
          <List.Item >
            <List.Icon name='file' />
            <List.Content>
              <List.Header>
                <Checkbox label={item.displayName} checked={this.state.itemState[parentKey][idx]} onChange={
                  (event: React.FormEvent<HTMLInputElement>, checkProps: CheckboxProps) => {
                    this.onItemCheckCBox(parentKey, checkProps, item);
                  }} />
                  {this.checkRecommend(item)}
              </List.Header>
              <List.Description style={{ marginLeft: '26px' }}>
                {this.getItemDesc(item)}
              </List.Description>
            </List.Content>
          </List.Item>
        </List.List>,
      );
    });

    return elems;
  }

  private checkRecommend(item: MetaSensorDisplayItem): JSX.Element {
    if (item.default) {
      return <Rating icon='star' defaultRating={3} maxRating={3} clearable={false} disabled={true} style={{marginLeft:10}}/>;
    }
    return null;
  }

  private getItemDesc(item: MetaSensorDisplayItem): string {
    if (item.default) {
      return '(Recommend) ' + item.key;
    }
    return item.key;
  }

  public render(): JSX.Element {
    return (
      <Container fluid>
        <List selection>
          {this.ViewSensorItemType()}
        </List>
      </Container>
    );
  }

  private sortItemMap(): void {

    if (this.itemMap.size > 0) {
      return;
    }

    for (let displayItem of this.props.sensorDisplayItems) {

      if (this.itemMap.has(displayItem.itemType.id)) {
        this.itemMap.get(displayItem.itemType.id).metaSensorDisplayItems.push(displayItem);
      } else {
        let item: TreeItem = { metaSensorDisplayItems: null, metaSensorItemType: null };
        item.metaSensorDisplayItems = [];
        item.metaSensorDisplayItems.push(displayItem);
        item.metaSensorItemType = displayItem.itemType;
        this.itemMap.set(displayItem.itemType.id, item);
      }
    }

  }

  private onItemClick = (parentKey: number, idx: number) => {
    let newItemState: Map<number, Array<boolean>> = _.clone(this.state.itemState);
    newItemState[parentKey][idx] = !newItemState[parentKey][idx];
    this.setState({ itemState: newItemState });
  }

  private onItemCheckCBox = (parentKey: number, checkProps: CheckboxProps, item: MetaSensorItem) => {
    if (this.selectedItemMap.get(parentKey) === undefined) {
      this.selectedItemMap.set(parentKey, []);
    }
    if (checkProps.checked) {
      this.selectedItemMap.get(parentKey).push(item);
    } else {
      let index: number = this.selectedItemMap.get(parentKey).indexOf(item);
      if (index >= 0) {
        this.selectedItemMap.get(parentKey).splice(index, 1);
      }
      if (this.selectedItemMap.get(parentKey).length <= 0) {
        let newCheckState: Map<number, boolean> = _.clone(this.state.categoryCheckState);
        newCheckState[parentKey] = false;
        this.setState({ categoryCheckState: newCheckState });
      }
    }
    let sd = this.props.getSensor();
    sd.sensorItemMap = this.selectedItemMap;
    this.props.setSensor(sd);
  }

  private onTypeClick = (key: number) => {
    let newCategoryState: Map<number, boolean> = _.clone(this.state.categoryState);
    newCategoryState[key] = !newCategoryState[key];
    this.setState({ categoryState: newCategoryState });
  }

  private onTypeCheckCBox = (key: number, checkProps: CheckboxProps, data: TreeItem) => {
    this.state.categoryCheckState[key] = checkProps.checked;
    let newItemState: Map<number, Array<boolean>> = _.clone(this.state.itemState);
    if (newItemState[key] === undefined) {
      newItemState[key] = new Array(data.metaSensorDisplayItems.length).fill(false);
    }
    newItemState[key].fill(checkProps.checked);
    if (checkProps.checked) {
      this.selectedItemMap.set(key, _.clone(data.metaSensorDisplayItems));
    } else {
      this.selectedItemMap.get(key).length = 0;
    }
    this.setState({ itemState: newItemState });

    let sd = this.props.getSensor();
    sd.sensorItemMap = this.selectedItemMap;
    this.props.setSensor(sd);
  }

  private checkUnCheckParent(metaSensorItemList: Array<MetaSensorItem>): boolean {
    if (metaSensorItemList.length <= 0) {
      return true;
    }
    return false;
  }

}

export default SensorItemTree;
