import { connect, Dispatch } from 'react-redux';
import {
    NotificationList,
    StateProps as StateProps,
    DispatchProps as DispatchProps,
} from './components/NotificationList';
import { push as routerPush } from 'react-router-redux';
import * as asyncRequestActions from '@overflow/commons/redux/action/asyncRequest';
import Member from '@overflow/member/api/model/Member';
import * as readAllActions from '../redux/action/read_all';
import PageParams from '@overflow/commons/api/model/PageParams';
import Notification from '@overflow/notification/api/model/Notification';
import * as readUnconfirmedCntActions from '../redux/action/read_unconfirmed_count';
import * as readActions from '../redux/action/read';

export function mapStateToProps(state: any, props: any): StateProps {
    return {
        notiPage: state.allNotifications,
    };
}

export function mapDispatchToProps(dispatch: Dispatch<any>): DispatchProps {
    return {
        onReadAll: (member: Member, pageParams: PageParams) => {
            dispatch(asyncRequestActions.request('NotificationService', 'readAllByMember',
                                                 readAllActions.REQUEST, JSON.stringify(member), JSON.stringify(pageParams)));
        },
        onRedirectByUrl: (url: string) => {
            dispatch(routerPush(url));
        },
        onMarkAsRead: (noti: Notification) => {
            Promise.all([
                dispatch(asyncRequestActions.request('NotificationService', 'markAsRead',
                                                     readActions.REQUEST, JSON.stringify(noti))),
                dispatch(asyncRequestActions.request('NotificationService', 'readUnconfirmedCount',
                                                     readUnconfirmedCntActions.REQUEST, JSON.stringify(noti.member))),
            ]);
        },
    };
}

export default connect(mapStateToProps, mapDispatchToProps)(NotificationList);

