import { connect, Dispatch } from 'react-redux';
import {
    NotificationPopup,
    StateProps as StateProps,
    DispatchProps as DispatchProps,
} from './components/NotificationPopup';
import { push as routerPush } from 'react-router-redux';
import * as asyncRequestActions from '@overflow/commons/redux/action/asyncRequest';
import Member from '@overflow/member/api/model/Member';
import Notification from '@overflow/notification/api/model/Notification';
import * as readAllUnconfirmedActions from '../redux/action/read_all_unconfirmed';
import * as readUnconfirmedCntActions from '../redux/action/read_unconfirmed_count';
import * as readActions from '../redux/action/read';
import PageParams from '@overflow/commons/api/model/PageParams';

export function mapStateToProps(state: any, props: any): StateProps {
    return {
        notifications: state.notifications,
    };
}

export function mapDispatchToProps(dispatch: Dispatch<any>): DispatchProps {
    return {
        onReadTop: (member: Member, pageParams: PageParams) => {
            dispatch(asyncRequestActions.request('NotificationService', 'readAllByMember',
                                                 readAllUnconfirmedActions.REQUEST, JSON.stringify(member), JSON.stringify(pageParams)));
        },
        onRedirectAllNotifications: () => {
            dispatch(routerPush('/notifications/'));
        },
        onMarkAllAsRead: (member: Member, pageParams: PageParams) => {
            Promise.all([
                dispatch(asyncRequestActions.request('NotificationService', 'markAllAsRead',
                                                     readAllUnconfirmedActions.REQUEST, JSON.stringify(member),
                                                     JSON.stringify(pageParams))),
                dispatch(asyncRequestActions.request('NotificationService', 'readUnconfirmedCount',
                                                     readUnconfirmedCntActions.REQUEST, JSON.stringify(member))),
            ]);
        },
        onMarkAllAsUnread: (member: Member, pageParams: PageParams) => {
            Promise.all([
                dispatch(asyncRequestActions.request('NotificationService', 'markAllAsUnread',
                                                     readAllUnconfirmedActions.REQUEST, JSON.stringify(member),
                                                     JSON.stringify(pageParams))),
                dispatch(asyncRequestActions.request('NotificationService', 'readUnconfirmedCount',
                                                     readUnconfirmedCntActions.REQUEST, JSON.stringify(member))),
            ]);
        },
        onRedirectByUrl: (url: string) => {
            dispatch(routerPush(url));
        },
        onMarkAsRead: (noti: Notification) => {
            // dispatch(asyncRequestActions.request('NotificationService', 'markAsRead',
            //                                     readAllUnconfirmedActions.REQUEST, JSON.stringify(noti)));
            Promise.all([
                dispatch(asyncRequestActions.request('NotificationService', 'markAsRead',
                                                     readActions.REQUEST, JSON.stringify(noti))),
                dispatch(asyncRequestActions.request('NotificationService', 'readUnconfirmedCount',
                                                     readUnconfirmedCntActions.REQUEST, JSON.stringify(noti.member))),
            ]);
        },
    };
}

export default connect(mapStateToProps, mapDispatchToProps)(NotificationPopup);

