import * as React from 'react';
import {
    Container,
    List,
    Header,
    Button,
    Label,
} from 'semantic-ui-react';
import Page from '@overflow/commons/api/model/Page';
import Member from '@overflow/member/api/model/Member';
import Notification from '../../api/model/Notification';
import PageParams from '@overflow/commons/api/model/PageParams';

export interface StateProps {
    notifications: Page;
}

export interface DispatchProps {
    onReadTop(member: Member, pageParams: PageParams): void;
    onRedirectAllNotifications(): void;
    onMarkAllAsRead(member: Member, pageParams: PageParams): void;
    onMarkAllAsUnread(member: Member, pageParams: PageParams): void;
    onRedirectByUrl(url: string): void;
    onMarkAsRead(notification: Notification): void;
}

export type Props = StateProps & DispatchProps;

export interface State {
    show: boolean;
}

export class NotificationPopup extends React.Component<Props, State> {

    private countPerPage: number = 5;
    constructor(props: Props, context: State) {
        super(props, context);
        this.state = {
            show: true,
        };
    }
    public componentWillMount(): void {
        const member: Member = {
            id: 1,
        };
        const pageParams: PageParams = {
            sortCol: 'id',
            sortDirection: 'descending',
            countPerPage: String(this.countPerPage),
            pageNo: String(0),
        };
        this.props.onReadTop(member, pageParams);
    }

    public renderItems(): (JSX.Element | JSX.Element[]) {
        if (this.props.notifications === undefined || this.props.notifications.content.length === 0) {
            return <div>No user notifications found. </div>;
        }
        return this.props.notifications.content.map((item: Notification, index: number) => (
            <List.Item style={this.handleStyle(item)} key={index}
            onClick={this.handleSelect.bind(this, item)}  >
                <List.Icon name='bell outline' size='large' verticalAlign='middle' />
                <List.Content>
                    <List.Header>{item.title}</List.Header>
                    <List.Description>{item.message}</List.Description>
                </List.Content>
            </List.Item>
        ));
    }

    private handleStyle(noti: Notification): any {
        if(noti.confirmDate === null) {
            return {'backgroundColor':'green'};
        }
        return null;
    }

    public render(): JSX.Element {
        if(!this.state.show) {
            return null;
        }

        return (
            <Container fluid>
                <Label size='large' ribbon color='red'>Notifications</Label>
                <Label as='a' onClick={this.handleMark}>Mark all as read</Label>
                {/* for Testing */}
                {/* <Label as='a' onClick={this.handleUnMark}>Mark all as unread</Label> */}
                <List divided relaxed>
                    {this.renderItems()}
                    <List.Item onClick={this.handleViewAll}>
                        <List.Content>
                            <List.Description as='a'><b>View All</b></List.Description>
                        </List.Content>
                    </List.Item>
                </List>
            </Container>
        );
    }

    private handleSelect = (noti: Notification): void => {
        this.props.onMarkAsRead(noti);
        this.props.onRedirectByUrl(noti.url);
        this.setState({
            show: false,
        });
    }

    private handleMark = (): void => {
        const member: Member = {
            id: 1,
        };
        const pageParams: PageParams = {
            sortCol: 'id',
            sortDirection: 'descending',
            countPerPage: String(this.countPerPage),
            pageNo: String(0),
        };
        this.props.onMarkAllAsRead(member, pageParams);
    }

    private handleUnMark = (): void => {
        const member: Member = {
            id: 1,
        };
        const pageParams: PageParams = {
            sortCol: 'id',
            sortDirection: 'descending',
            countPerPage: String(this.countPerPage),
            pageNo: String(0),
        };
        this.props.onMarkAllAsUnread(member, pageParams);
    }

    private handleViewAll = (): void => {
        this.setState({
            show: false,
        });
        this.props.onRedirectAllNotifications();
    }
}
