import * as React from 'react';
import { Table, Button, Header, Container, Card, Image, Icon } from 'semantic-ui-react';

import Sensor from '@overflow/sensor/api/model/Sensor';
import SensorItem from '@overflow/sensor/api/model/SensorItem';
import Page from '@overflow/commons/api/model/Page';
import { Pager } from '@overflow/commons/react/component/Pager';
import PageParams from '@overflow/commons/api/model/PageParams';
import { SortTableHeaderRow } from '@overflow/commons/react/component/SortTableHeaderRow';
import { LoadChartLinux } from '@overflow/metric/react/components/chart/LoadChartLinux';

export interface StateProps {
  sensorId?: number;
  sensorItemList?: Page;
}

export interface DispatchProps {
  onReadAllBySensor?(sensor: Sensor, pageParams: PageParams): void;
}

export type Props = StateProps & DispatchProps;

export interface State {

}


export class SensorDetailItems extends React.Component<Props, State> {

  private countPerPage: number = 100000;
  private sortCol: string = 'id';
  private sortDirection: string = 'ascending';
  private pageNo: number = 0;

  constructor(props: Props, context: State) {
    super(props, context);
    this.state = {
      // sensorItemList: SensorItemJson,
    };

    this.getData();
  }

  private getData(): void {
    const pageParams: PageParams = {
      pageNo: String(this.pageNo),
      countPerPage: String(this.countPerPage),
      sortCol: this.sortCol,
      sortDirection: this.sortDirection,
    };
    this.props.onReadAllBySensor({ id: this.props.sensorId }, pageParams);
  }


  public render(): JSX.Element {
    if (this.props.sensorItemList === undefined) {
      return null;
    }
    // let colsmap: Map<string, string> = new Map();
    // colsmap.set('Type', 'item.itemType.name');
    // colsmap.set('Name', 'item.displayName');
    // colsmap.set('Unit', 'item.unit.mark');
    // colsmap.set('Description', 'item.description');

    return (
      <Container fluid>
        {this.renderSensorItemInfo()}

        {/* <Table celled sortable>
          <Table.Header>
            <SortTableHeaderRow columnMap={colsmap} onHeaderColumnClick={this.handleSort} />
          </Table.Header>

          <Table.Body>
            {
              this.props.sensorItemList.content ? this.props.sensorItemList.content.map((sensorItem: SensorItem, idx: number) => {
                return (
                  <Table.Row key={idx}>
                    <Table.Cell collapsing>{sensorItem.item.itemType.name}</Table.Cell>
                    <Table.Cell collapsing>{sensorItem.item.displayName}</Table.Cell>
                    <Table.Cell collapsing textAlign='center'>{sensorItem.item.unit.mark}</Table.Cell>
                    <Table.Cell>{sensorItem.item.description}</Table.Cell>
                  </Table.Row>
                );
              }) : ''
            }
          </Table.Body>
          <Table.Footer>
            <Table.Row>
              <Table.HeaderCell colSpan='4'>
                <Pager page={this.props.sensorItemList} onPageChange={this.handlePaging} />
              </Table.HeaderCell>
            </Table.Row>
          </Table.Footer>
        </Table> */}
      </Container>
    );
  }
  private renderSensorItemInfo(): JSX.Element {
    return this.props.sensorItemList.content.map((sensorItem: SensorItem, index: number) => (
      <Card fluid key={index}>
      <Card.Content>
        <Card.Header>{sensorItem.item.displayName}</Card.Header>
        <Card.Meta>
          <span className='date'>{sensorItem.item.description}</span>
        </Card.Meta>
        <Card.Description>{sensorItem.sensor.crawler.name}</Card.Description>
      </Card.Content>
      <Card.Content extra>
        {this.getChart(sensorItem)}
      </Card.Content>
    </Card>
    ));
  }
  private getChart(sensorItem: SensorItem): JSX.Element {
    return <LoadChartLinux itemKey={'cpu.load.all'} />;
  }

  // private handleSort = (col: string, direction: string): void => {
  //   console.log(col + ' || direction:' + direction);
  //   this.sortCol = col;
  //   this.sortDirection = direction;
  //   this.getData();
  // }
  // private handlePaging = (pageNo: number) => {
  //   this.pageNo = pageNo;
  //   const pageParams: PageParams = {
  //     pageNo: String(this.pageNo),
  //     countPerPage: String(this.countPerPage),
  //     sortCol: this.sortCol,
  //     sortDirection: this.sortDirection,
  //   };
  //   this.props.onReadAllBySensor({ id: this.props.sensorId }, pageParams);
  // }

}
