import * as React from 'react';
import {
  Table, Header, Container, Form, Checkbox, Button, Rating,
  List, Icon, CheckboxProps,
} from 'semantic-ui-react';
import * as _ from 'lodash';


export interface StateProps {
  ItemTreeList: Array<ItemTreeObj>;
}

export interface DispatchProps {
}

export type Props = StateProps & DispatchProps;

export interface State {
  dropState: Map<string, boolean>;
  // checkBoxState: Map<string, boolean>;
  checkBoxChildrenState: Map<string, Map<string, boolean>>;
}

export interface ItemTreeObj {
  icon?: string;
  isCheckBox?: boolean;
  label: string;
  children?: Array<ItemTreeObj>;
}

export class ItemTree extends React.Component<Props, State> {

  constructor(props: Props, context: State) {
    super(props, context);

    this.state = {
      dropState: new Map(),
      // checkBoxState: new Map(),
      checkBoxChildrenState: new Map(),
    };

  }

  private onDropClick = (key: string) => {
    let newDropState: Map<string, boolean> = _.clone(this.state.dropState);
    newDropState[key] = !newDropState[key];
    this.setState({ dropState: newDropState });
  }

  private getKey(pKey: string, idx: number): string {
    return pKey + '-' + String(idx);
  }

  private reverseChildrenCheck(childItemList: Array<ItemTreeObj>, pKey: string, checkProps: CheckboxProps,
                               checkBoxState: Map<string, Map<string, boolean>>): void {

    if(childItemList === undefined || childItemList === null || childItemList.length <= 0) {
      return;
    }

    if (checkBoxState.get(pKey) === undefined) {
      checkBoxState.set(pKey, new Map());
    }

    let currentKey: string = '';
    childItemList.map((childItem: ItemTreeObj, childIdx: number) => {
      currentKey = this.getKey(pKey, childIdx);
      checkBoxState.get(pKey).set(currentKey, checkProps.checked);
      this.reverseChildrenCheck(childItem.children, currentKey, checkProps, checkBoxState);
    });
  }

  private onCheckBox = (pKey: string, key: string, checkProps: CheckboxProps, data: ItemTreeObj) => {
    // this.state.checkBoxState[key] = checkProps.checked;
    let newCheckBoxChildrenState: Map<string, Map<string, boolean>> = _.clone(this.state.checkBoxChildrenState);

    if(newCheckBoxChildrenState.get(pKey) === undefined) {
      newCheckBoxChildrenState.set(pKey, new Map());
    }
    newCheckBoxChildrenState.get(pKey).set(key, checkProps.checked);

    this.reverseChildrenCheck(data.children, key, checkProps, newCheckBoxChildrenState);

    // newCheckBoxChildrenState[key].forEach((mValue: boolean, mkey: string) => {
    //   newCheckBoxChildrenState[mkey] = checkProps.checked;
    // });
    // newCheckBoxChildrenState[key].fill(checkProps.checked);
    // if (checkProps.checked) {
    //   this.selectedItemMap.set(key, _.clone(data.children));
    // } else {
    //   this.selectedItemMap.get(key).length = 0;
    // }
    this.setState({ checkBoxChildrenState: newCheckBoxChildrenState });

    // let sd = this.props.getSensor();
    // sd.sensorItemMap = this.selectedItemMap;
    // this.props.setSensor(sd);
  }

  private checkCheckBox(key: string, pKey: string): boolean {

    let resBool: boolean = false;

    // if (this.state.checkBoxState[key]) {
    //   resBool = true;
    // }

    // if (pKey !== '') {

    let childrenState: Map<string, boolean> = this.state.checkBoxChildrenState.get(pKey);

    if (childrenState !== undefined) {
      let b: boolean = childrenState.get(key);
      resBool = b;
    }

    // }
    return resBool;
  }


  public startRender(): JSX.Element[] {

    // if (this.props.ItemTreeList === undefined || this.props.ItemTreeList.length <= 0) {
    //   return null;
    // }

    let elems: JSX.Element[];

    elems = this.itemRender(this.props.ItemTreeList, 'r');

    return elems;
  }

  public itemRender(itemList: Array<ItemTreeObj>, pKeyStr: string): JSX.Element[] {

    if (itemList === undefined || itemList === null || itemList.length <= 0) {
      return null;
    }

    let elems: JSX.Element[] = new Array();

    itemList.map((item: ItemTreeObj, idx: number) => {
      let nkeyStr: string = this.getKey(pKeyStr, idx);
      let trueStr: string;
      let falseStr: string;
      if (item.icon) {
        let iconStr: string = item.icon;
        let cIdx: number = iconStr.indexOf('|');
        trueStr = iconStr.substring(0, cIdx);
        falseStr = iconStr.substring(cIdx + 1);
      }
      elems.push(
        <List.Item key={nkeyStr}>
          {item.icon ?
            <List.Icon name={this.state.dropState[nkeyStr] ? trueStr : falseStr}
              onClick={this.onDropClick.bind(this, nkeyStr)} /> :
            ''}
          <List.Content>
            <List.Item>
              <List.Header>
                {item.isCheckBox === false ? item.label :
                  <Checkbox label={item.label} checked={this.checkCheckBox(nkeyStr, pKeyStr) ? true : false} onChange={
                    (event: React.FormEvent<HTMLInputElement>, checkProps: CheckboxProps) => {
                      this.onCheckBox(pKeyStr, nkeyStr, checkProps, item);
                    }} />}
              </List.Header>
              <List.Description style={{ marginLeft: '26px' }}></List.Description>
              {this.state.dropState[nkeyStr] ?
                ''
                :
                <List.List>
                  {this.itemRender(item.children, nkeyStr)}
                </List.List>
              }
            </List.Item>
          </List.Content>
        </List.Item>,
      );
    });


    return elems;
  }


  public render(): JSX.Element {
    return (
      <Container>
        <List>
          {this.startRender()}
        </List>
      </Container>
    );
  }

}
